#!/usr/bin/env python3
import argparse
import array
import errno
import fcntl
import importlib.metadata
import json
import os
import pty
import re
import select
import signal
import socket
import struct
import sys
import termios
import threading
import tty
from pathlib import Path
from typing import Dict
from typing import List

from .presenter_view import run_presenter_view

try:
    VERSION = importlib.metadata.metadata("itsalive")["Version"]
except Exception:
    VERSION = "0.0.0"


def strip_ansi(text: bytes) -> bytes:
    """Strip all ANSI sequences from `text`."""
    # Python Ansi Regex originally sourced from - https://stackoverflow.com/questions/14693701/how-can-i-remove-the-ansi-escape-sequences-from-a-string-in-python
    # OSC Regex implementation adapted from - https://unix.stackexchange.com/questions/14684/removing-control-chars-including-console-codes-colours-from-script-output
    ansi_escape = re.compile(
        rb"""
       (?:\x1B\]|\x9d).*(?:\x1B\\|[\a\x9c]) # OSC
       |   # or
       (?: # either 7-bit C1, two bytes, ESC Fe (omitting CSI)
           \x1B
           [@-Z\\-_]
       |   # or a single 8-bit byte Fe (omitting CSI)
           [\x80-\x9A\x9C-\x9F]
       |   # or CSI + control codes
           (?: # 7-bit CSI, ESC [
               \x1B\[
           |   # 8-bit CSI, 9B
               \x9B
           )
           [0-?]*  # Parameter bytes
           [ -/]*  # Intermediate bytes
           [@-~]   # Final byte
       )
    """,
        re.VERBOSE,
    )
    return ansi_escape.sub(b"", text)


def load_file(filename: Path) -> List[str]:
    """Load the commands from a file."""
    with filename.expanduser().open("r") as infile:
        cmd_list = infile.read().rstrip("\n").split("\n")

    return cmd_list


def load_commands(filename: Path) -> List[str]:
    """Load commands from a file, processing them as required."""
    new_list = []
    cmd_list = load_file(filename)
    filename = filename.expanduser()
    for command in cmd_list:
        include = re.match(r"^##@include\s+(?P<filename>.*?)$", command)
        if include:
            include_file_name = Path(
                os.path.join(
                    filename.parent, Path(include.group("filename")).expanduser()
                )
            ).resolve()
            if not include_file_name.exists():
                sys.exit("%s does not exist." % include_file_name)
            new_list.extend(load_file(include_file_name))
        else:
            new_list.append(command)

    return new_list


class PresenterServer(threading.Thread):
    """
    The presenter server.

    This class implements a server to which clients can connect to receive
    information about the current state of the presentation.
    """

    daemon = True

    def __init__(self, commands, address, port):
        threading.Thread.__init__(self)
        self.clients = []
        self.commands = commands

        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.sock.bind((address, port))
        self.sock.listen()

        self._command_index = 0
        self._command_offset = 0

    def _send_to_client(self, client, data):
        """Send an update to a single client."""
        try:
            client.send(json.dumps(data).encode() + b"\n")
        except BrokenPipeError:
            pass

    def _send_to_all(self, data):
        """Send an update to all connected clients."""
        for client in self.clients:
            self._send_to_client(client, data)

    def _make_initial_message(self) -> Dict[str, str]:
        return {"command": "initial", "data": self.commands}

    def _make_cursor_update_message(self) -> Dict[str, str]:
        return {
            "command": "cursor",
            "index": self._command_index,
            "offset": self._command_offset,
        }

    def _handle_connections(self):
        """Handle new connections."""
        while True:
            connection, client_address = self.sock.accept()
            self._send_to_client(connection, self._make_initial_message())
            self._send_to_client(connection, self._make_cursor_update_message())
            self.clients.append(connection)

    def update_cursor(self, command_index: int, command_offset: int) -> None:
        # This is unsynchronized because I don't anticipate any race conditions,
        # since this is the only place where these variables get written to.
        self._command_index = command_index
        self._command_offset = command_offset
        self._send_to_all(self._make_cursor_update_message())

    def send_command(self, command: str) -> None:
        self._send_to_all({"command": command})

    def run_server(self):
        """Run the TCP server to accent connections on a different thread."""
        threading.Thread(target=self._handle_connections, daemon=True).start()

    def shutdown(self):
        self.send_command("exit")
        self.sock.shutdown(socket.SHUT_RDWR)
        self.sock.close()


class Raw:
    def __init__(self, fd):
        self.fd = fd
        self.restore = False

    def __enter__(self):
        try:
            self.mode = tty.tcgetattr(self.fd)
            tty.setraw(self.fd)
            self.restore = True
        except tty.error:  # This is the same as termios.error
            pass

    def __exit__(self, type, value, traceback):
        if self.restore:
            tty.tcsetattr(self.fd, tty.TCSAFLUSH, self.mode)


class Player:
    def __init__(
        self,
        command_filename: Path,
        env=os.environ.copy(),
        speed=1,
        address=None,
        port=None,
    ):
        self.master_fd = None
        self.paused = False
        # Which command in the list we're at.
        self.command_index = 0
        # Where in the command we currently are.
        self.command_offset = 0
        self.env = env
        self.speed = max(0, int(speed))

        # Read the commands from the file into a list of one per line.
        commands = load_commands(command_filename)
        # Add a blank command to the end, so we don't exit prematurely.
        self.commands = commands + [""]

        self.ps = PresenterServer(self.commands, address=address, port=port)
        self.ps.run_server()

    @property
    def _current_command(self) -> str:
        return self.commands[self.command_index]

    @property
    def _is_first_command(self) -> bool:
        """Return whether this is the first command."""
        return self.command_index == 0

    @property
    def _is_last_command(self) -> bool:
        """Return whether this is the last command."""
        return self.command_index >= len(self.commands) - 1

    def _reset_command(self):
        """Reset the current command to the beginning and send it."""
        self.command_offset = 0

        self.ps.update_cursor(self.command_index, self.command_offset)

    def _set_paused(self, paused: bool):
        """Pause or resume playback."""
        self.paused = paused
        if self.paused:
            self.ps.send_command("pause")
        else:
            self.ps.send_command("resume")

    def _next_command(self):
        """Change the command index to the next command."""
        while True:
            if self._is_last_command:
                self.ps.shutdown()
                sys.exit()

            self.command_index += 1

            # Initiate a command-file-driven pause. When '##@pause' is encountered
            # automatic playback will be paused. The next command will be ready
            # for use when manually resumed.
            #
            # ##@pause is executed following a command with carriage return. When
            # using the skip command combinations (Ctrl+f / Ctrl+g), this is ignored.
            if self._current_command == ("##@pause") and not self.skip_keypress:
                self._set_paused(True)

            # Skip over comments.
            if not self._current_command.startswith("##"):
                self._reset_command()
                break

    def _previous_command(self):
        """Change the command index to the previous command."""
        while True:
            self.command_index = max(0, self.command_index - 1)

            if not self._current_command.startswith("##"):
                self._reset_command()
                break

    def _set_pty_size(self):
        """Set the window size of the child pty based on the window size of our own controlling terminal."""
        # Get the terminal size of the real terminal, set it on the pseudoterminal.
        if os.isatty(pty.STDOUT_FILENO):
            buf = array.array("h", [0, 0, 0, 0])
            fcntl.ioctl(pty.STDOUT_FILENO, termios.TIOCGWINSZ, buf, True)
        else:
            buf = array.array("h", [24, 80, 0, 0])

        fcntl.ioctl(self.master_fd, termios.TIOCSWINSZ, buf)

    def _write_stdout(self, data):
        """Write to stdout as if the child process had written the data."""
        os.write(pty.STDOUT_FILENO, data)

    def _handle_master_read(self, data):
        """Handle new data on child process stdout."""
        self._write_stdout(data)

    def _write_master(self, data, passthrough=False):
        """Write to the child process from its controlling terminal."""
        command_end = False
        if passthrough:
            while data:
                n = os.write(self.master_fd, data)
                data = data[n:]
            return

        data = strip_ansi(data)
        if not data:
            # Sometimes we get pure ANSI sequences, just ignore those.
            return

        next_char = self._current_command[
            self.command_offset : self.command_offset + self.speed
        ]
        if not next_char:
            if b"\r" not in data and b"\n" not in data:
                return
            else:
                next_char = "\n"
                command_end = True

        os.write(self.master_fd, next_char.encode())
        self.command_offset += self.speed
        if command_end:
            self._next_command()
        else:
            self.ps.update_cursor(self.command_index, self.command_offset)

    def _handle_stdin_read(self, data):
        """Handle new data on child process stdin."""
        # Register for when a skip key (Ctrl+f / Ctrl+g) is used
        self.skip_keypress = False

        if data == b"\x10":  # Ctrl+p
            self._set_paused(True)
            return
        elif data == b"\x12":  # Ctrl+r
            self._set_paused(False)
            return

        # If we aren't paused, accept commands.
        if self.paused:
            self._write_master(data, passthrough=True)
        else:
            if data == b"\x04":  # Ctrl+d
                self.ps.shutdown()
                sys.exit("Abort!")
            elif data == b"\x05":  # Ctrl+e
                # Type the rest of the command by setting the speed to the entire command,
                # then emitting a single character so it types everything up to the newline.
                speed_backup = self.speed
                self.speed = len(self._current_command.encode())
                self._write_master(b"a")
                self.speed = speed_backup
            elif data == b"\x15":  # Ctrl+u
                # Pass it through.
                self._write_master(data, passthrough=True)
                self._reset_command()
            elif data == b"\x06":  # Ctrl+f
                # Go to the next command.
                if not self._is_last_command:
                    self.skip_keypress = True
                    self._next_command()
            elif data == b"\x07":  # Ctrl+g
                # Go to the previous command.
                self.skip_keypress = True
                self._previous_command()
            else:
                self._write_master(data)

    def _signals(self, signal_list):
        old_handlers = []
        for sig, handler in signal_list:
            old_handlers.append((sig, signal.signal(sig, handler)))
        return old_handlers

    def _copy(self, signal_fd):
        """
        Run the main select loop.

        Passes control to _master_read() or _stdin_read() when new data arrives.
        """
        fds = [self.master_fd, pty.STDIN_FILENO, signal_fd]

        while True:
            try:
                rfds, wfds, xfds = select.select(fds, [], [])
            except OSError as e:  # Python >= 3.3
                if e.errno == errno.EINTR:
                    continue
            except select.error as e:  # Python < 3.3
                if e.args[0] == 4:
                    continue

            if self.master_fd in rfds:
                data = os.read(self.master_fd, 1024)
                if not data:  # Reached EOF.
                    fds.remove(self.master_fd)
                else:
                    self._handle_master_read(data)

            if pty.STDIN_FILENO in rfds:
                data = os.read(pty.STDIN_FILENO, 1024)
                if not data:
                    fds.remove(pty.STDIN_FILENO)
                else:
                    self._handle_stdin_read(data)

            if signal_fd in rfds:
                data = os.read(signal_fd, 1024)
                if data:
                    signals = struct.unpack("%uB" % len(data), data)
                    for sig in signals:
                        if sig in [
                            signal.SIGCHLD,
                            signal.SIGHUP,
                            signal.SIGTERM,
                            signal.SIGQUIT,
                        ]:
                            os.close(self.master_fd)
                            return
                        elif sig == signal.SIGWINCH:
                            self._set_pty_size()

    def play(self):
        """Start the player."""
        self._reset_command()

        command = ["sh", "-c", os.environ.get("SHELL") or "sh"]
        pid, self.master_fd = pty.fork()

        if pid == pty.CHILD:
            os.execvpe(command[0], command, self.env)

        pipe_r, pipe_w = os.pipe()
        flags = fcntl.fcntl(pipe_w, fcntl.F_GETFL, 0)
        flags = flags | os.O_NONBLOCK
        flags = fcntl.fcntl(pipe_w, fcntl.F_SETFL, flags)

        signal.set_wakeup_fd(pipe_w)

        old_handlers = self._signals(
            map(
                lambda s: (s, lambda signal, frame: None),
                [
                    signal.SIGWINCH,
                    signal.SIGCHLD,
                    signal.SIGHUP,
                    signal.SIGTERM,
                    signal.SIGQUIT,
                ],
            )
        )

        self._set_pty_size()

        with Raw(pty.STDIN_FILENO):
            try:
                # The main loop function, where all the typing work happens.
                self._copy(pipe_r)
            except (IOError, OSError):
                pass

        self._signals(old_handlers)

        os.waitpid(pid, 0)


def main():
    parser = argparse.ArgumentParser(description="It's a live.")
    parser.add_argument(
        "command_file",
        metavar="command_file",
        type=str,
        help="The command file to read.",
    )
    parser.add_argument(
        "--address",
        dest="address",
        type=str,
        default="127.0.0.1",
        help="The address to listen on/connect to for presenter view (default: %(default)s).",
    )
    parser.add_argument(
        "--port",
        dest="port",
        type=int,
        default=5345,
        help="The port to listen on/connect to for presenter view (default: %(default)s).",
    )
    parser.add_argument(
        "-s",
        "--speed",
        dest="speed",
        type=int,
        default=1,
        help="How many characters to type out when you type one (default: %(default)s).",
    )
    parser.add_argument(
        "-c", "--clear", action="store_true", help="Clear the screen before starting"
    )
    parser.add_argument("--version", action="version", version="%(prog)s " + VERSION)

    args = parser.parse_args()
    if args.clear:
        # From https://stackoverflow.com/a/52949759/28196.
        print("{}[2J{}[;H".format(chr(27), chr(27)))

    if args.command_file == "presenter_view":
        run_presenter_view(args.address, args.port)
    else:
        Player(
            Path(args.command_file),
            speed=args.speed,
            address=args.address,
            port=args.port,
        ).play()


if __name__ == "__main__":
    main()



"""
Implement the presenter view machinery.

Presenter view runs on one main thread and one background thread. The main thread sets up the curses interface, and the
background thread connects to the presentation server to receive updates. The threads are set up this way so the curses
thread can shut down when it receives a Ctrl+d keystroke.
"""
import curses
import json
import socket
import sys
import threading
from typing import Any
from typing import Dict
from typing import List


class PresenterView:
    def __init__(self, address: str, port: int):
        self.address = address
        self.port = port

        self.commands: List[str] = []
        # Set this to None so we print the first command, even if it's at 0.
        self.command_index = None
        self.command_offset = 0
        self.paused = False
        self.exiting = False

        # An event to signal that curses is done setting up, as otherwise there is corruption when the socket tries to
        # write to the output before curses is fully set up.
        self.curses_done = threading.Event()

    def process_command(self, command: Dict[str, Any]):
        """Process a command that came in through the socket."""
        if command["command"] == "initial":
            self.commands = command["data"]
        elif command["command"] == "exit":
            self.exiting = True
        elif command["command"] == "cursor":
            self.command_index = command["index"]
            self.command_offset = command["offset"]
            self.draw_screen()
        elif command["command"] == "pause":
            self.paused = True
            self.draw_screen()
        elif command["command"] == "resume":
            self.paused = False
            self.draw_screen()

    def connect(self):
        """Connect to the specified presentation server."""
        self.sock = socket.socket()
        try:
            self.sock.connect((self.address, self.port))
        except Exception as e:
            self.sock.close()
            sys.exit(
                "There was an error connecting to the presentation server. Please double-check the address and\n"
                "port, and check that no firewall rules are blocking the connection.\n\nThe error was:\n"
                + str(e)
            )

    def run_client(self):
        """Run the TCP client loop."""
        # Wait for curses to set up.
        self.curses_done.wait()

        BUF_SIZE = 2**12
        data = b""
        while True:
            d = self.sock.recv(BUF_SIZE)

            if not d:
                break

            data += d

            if b"\n" not in d:
                continue

            for line in data.decode().strip("\n").split("\n"):
                self.process_command(json.loads(line))
            data = b""
        self.sock.close()

    def draw_screen(self):
        """Draw the screen based on the current status."""
        self.screen.clear()
        y, x = self.screen.getmaxyx()
        self.screen.border()

        top_margin = 1
        left_margin = 1

        midheight = y // 2

        first_command_row = max(top_margin, midheight - self.command_index)
        last_command_row = min(
            y - top_margin * 2, midheight + len(self.commands) - self.command_index
        )

        first_command_offset = max(0, self.command_index - midheight + top_margin)

        # Draw the commands themselves.
        for c, i in enumerate(
            range(first_command_row, last_command_row), start=first_command_offset
        ):
            formatted_command = repr(self.commands[c].rstrip("\n"))[1:-1].encode()

            # Draw comments in green.
            if self.commands[c].startswith("##"):
                color = 2
            else:
                color = 0

            # Draw the sidebar |.
            self.screen.addstr(i, left_margin + 1, "|", curses.color_pair(3))

            # Draw the command in its entirety.
            self.screen.addstr(
                i,
                left_margin + 3,
                formatted_command[: max(0, x - left_margin * 2 - 3)],
                curses.color_pair(color),
            )
            # Update the command color according to keypresses for the current command.
            if c == self.command_index:
                # Set the live_color_offset_end for live command coloring.
                #
                # If it's a very long command that exceeds the terminal window width,
                # perform live updates until we get to the viewable command end
                # according to the terminal width.
                #
                # Otherwise color from the command start to self.command_offset.
                if self.command_offset > max(0, x - left_margin * 2 - 3):
                    live_color_offset_end = max(0, x - left_margin * 2 - 3)
                else:
                    live_color_offset_end = self.command_offset
                self.screen.addstr(
                    i,
                    left_margin + 3,
                    formatted_command[:live_color_offset_end],
                    curses.color_pair(4),
                )

        # Draw the current command marker.
        self.screen.addstr(midheight, left_margin, "->", curses.color_pair(1))

        self.screen.refresh()

        if self.paused:
            # Draw the "Paused" window.
            win = curses.newwin(5, 20, y // 2 - 2, x // 2 - 10)
            win.border()
            win.addstr(2, 7, "PAUSED", curses.color_pair(1))
            win.refresh()

    def curses_init(self, stdscr):
        """Initialize curses."""
        # Set up curses.
        self.screen = stdscr
        curses.curs_set(0)
        self.screen.erase()
        self.screen.refresh()

        # Set up colors.
        curses.use_default_colors()
        curses.init_pair(1, curses.COLOR_RED, -1)
        curses.init_pair(2, curses.COLOR_GREEN, -1)
        curses.init_pair(3, curses.COLOR_BLUE, -1)
        curses.init_pair(4, curses.COLOR_CYAN, -1)

        # Signal to the socket that we are done setting up curses.
        self.curses_done.set()

    def curses_main(self, stdscr):
        """Set up curses and run the main loop."""
        self.curses_init(stdscr)
        while True:
            key = self.screen.getkey()
            if key == "KEY_RESIZE":
                self.draw_screen()
            elif key == "\x04" or self.exiting:
                sys.exit("Bye!")

    def run(self):
        print("Connecting to the presenter server...")
        self.connect()
        threading.Thread(target=self.run_client, daemon=True).start()
        curses.wrapper(self.curses_main)


def run_presenter_view(address: str, port: int):
    PresenterView(address=address, port=port).run()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['itsalive']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['itsalive = itsalive.cli:main']}

setup_kwargs = {
    'name': 'itsalive',
    'version': '0.2.4',
    'description': 'Autopilot for live demos.',
    'long_description': 'It\'s a Live\n===========\n\n![](misc/logo.png)\n\nIt\'s a Live is a utility that helps you make live coding demos less error-prone by taking the "live" component and\nkilling it.\n\nIt\'s a Live lets you write commands and keystrokes in a file, which it will then read and open a new terminal for you.\nEvery time you press a key, It\'s a Live will write one character from the file into the terminal, making it look like\nyou\'re typing every single command with the practiced ease of a consummate professional.\n\n\nWhat it looks like\n------------------\n\nThis is what it looks like:\n\n![](misc/screenshot.png)\n\nThe typing terminal is on the left and the presenter view is on the right.\n\nHere\'s a screencast of It\'s a Live in operation. Keep in mind that the presenter is pressing keys randomly after the\nprogram starts:\n\n[![asciicast](https://asciinema.org/a/308560.svg)](https://asciinema.org/a/308560)\n\n\nInstallation\n------------\n\nYou can install It\'s a Live with pip:\n\n```\npip install itsalive\n```\n\nThat\'s about it.\n\n\nUsage\n-----\n\nUsing It\'s a Live is pretty simple:\nJust write some keystrokes or commands in a file and run `itsalive` with it:\n\n```\nitsalive <command_file>\n```\n\nIt\'s a Live will wait for you to press a key, and, when you do, it will instead emit one character from the command\nfile. This way, you can type whatever old crap and it will look perfectly rehearsed, every time, with no backspaces\n(unless you add them in).  It will also wait for you to press Enter at the end of commands, so you will never skip\nahead to the next command by mistake.\n\nWhat\'s more, It\'s a Live is actually running the commands you\'re typing, so you have full interoperability with other\nprograms.\n\nIt\'s a Live also supports various commands:\n\n* `Ctrl+d` will immediately terminate the playback.\n* `Ctrl+p` will pause automatic playback and give you control of the terminal. This is useful for doing actually live\n  stuff, just make sure to leave everything in a state so that playback can resume later.\n* `Ctrl+r` will resume playback.\n* `Ctrl+f` will skip forward to the next command.\n* `Ctrl+g` will skip back to the previous command.\n* `Ctrl+u` will send a `Ctrl+u` keystroke (wiping anything on to the left of the cursor) and rewind the current command.\n* `Ctrl+e` will type out the current command in its entirety.\n\n\nPresenter view\n--------------\n\nIt\'s a Live supports a presenter view, which will show the next command to be typed. To launch the presenter view, start\nthe presentation and run, on a separate terminal:\n\n```\nitsalive presenter_view\n```\n\nIf you want to leave yourself notes, you can add comments to the command file. Comments must start with `##` as the\nfirst thing on the line, and they will not be typed. They will only be shown above the command in the presenter view.\n\n\nSpecial commands\n----------------\n\nThere are special commands you can add to your files. The line must start with them, with no spaces before them.\n\n---\n\n**`##@include <filename>`:** This inserts the contents of `<filename>` at the position of the `include` command. The\nfile will be typed out, as if you had pasted it in the commands file.\n\nExample: `##@include somefile.py`.\n\n**`##@pause`:** Pause automatic playback.  This is useful when you\'re executing a command and know that the command that follows will be manually interacted with.  This functionality is only available post command execution and is ignored when using skip key combinations (Ctrl+f / Ctrl+g).\n\n---\n\n\nLicense\n-------\n\nIt\'s a Live is licensed under the GPL v3 or any later version.\n\n\nAcknowledgements\n---------------\n\nI would like to thank my bestie Ian Cromwell, without whom this project would be nameless.\n\n# Changelog\n\n\n## v0.2.4 (2022-08-11)\n\n### Features\n\n* Feat: Add `##@pause` command. [James Spurin]\n\n* Colored presenter live updates. [James Spurin]\n\n### Fixes\n\n* Fix description. [Stavros Korokithakis]\n\n\n## v0.2.3 (2022-02-28)\n\n### Fixes\n\n* Don\'t eat characters when OSC codes are emitted. [James Spurin]\n\n\n## v0.2.2 (2021-09-30)\n\n### Features\n\n* Add "--clear" flag. [Stavros Korokithakis]\n\n### Fixes\n\n* Don\'t freeze in case a presentation packet is exactly as large as the buffer size. [Stavros Korokithakis]\n\n\n## v0.2.1 (2020-03-10)\n\n### Features\n\n* Add command gutter so empty lines are more obvious. [Stavros Korokithakis]\n\n### Fixes\n\n* Disallow jumping past the last command (and ending playback) [Stavros Korokithakis]\n\n* Don\'t skip over the last empty command. [Stavros Korokithakis]\n\n* Fix race condition where the socket thread printed things to the screen before curses was set up. [Stavros Korokithakis]\n\n* Add the missing `--address` argument. [Stavros Korokithakis]\n\n\n## v0.2.0 (2020-03-09)\n\n### Features\n\n* Add curses-based presenter view. [Stavros Korokithakis]\n\n* Add Ctrl+r as a resumption shortcut. [Stavros Korokithakis]\n\n### Fixes\n\n* Only update the presenter view if the command changes. [Stavros Korokithakis]\n\n* Change Ctrl+b to Ctrl+g, as the former clashed with tmux. [Stavros Korokithakis]\n\n\n## 0.1.3 (2020-03-04)\n\n### Features\n\n* Feat: Add `##@include` directive. [Stavros Korokithakis]\n\n\n',
    'author': 'Stavros Korokithakis',
    'author_email': 'hi@stavros.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/stavros/itsalive',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
